// $Id: getreadydisplay.cc,v 1.5 2006/08/05 09:08:59 matthew Exp $

//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "getreadydisplay.h"
#include "SDL_image.h"
#include <cstdio>
#include <iostream>
#include <string>




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GetReadyDisplay::GetReadyDisplay(SDL_Surface * s)
{
    screen = s;
    
    // load font - this surface will be freed by the SFont object's destructor
    font_gfx = SDL_DisplayFormatAlpha(IMG_Load( PKG_DATA_DIR "/images/fonts/small_font.png" ));
    SDL_SetAlpha(font_gfx, SDL_SRCALPHA|SDL_RLEACCEL, 0);
    
    // initialize SFont object
    my_font.init(font_gfx);
    
    // load background
    background = SDL_DisplayFormat(IMG_Load( PKG_DATA_DIR "/images/getready/get_ready_bg.png" ));
    
} // FS::GetReadyDisplay::GetReadyDisplay()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GetReadyDisplay::~GetReadyDisplay()
{
    SDL_FreeSurface(background);

} // FS::GetReadyDisplay::~GetReadyDisplay() 




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::GetReadyDisplay::start_draw(int gt, int lev)
{
    start_time = gt;
    
    SDL_BlitSurface(background, NULL, screen, NULL);
    
    std::string message;
    char digits[4]; // allows for 999 levels!
    
    sprintf(digits, "%d", lev);
    message = "About to start level ";
    message += digits;
    message += " . . .";
    
    my_font.writeCenter(screen, message.c_str(), 50);
    my_font.writeCenter(screen, "Get Ready!",90);
    // FIXME: second arg shouldn't be hard-coded.
    my_font.writeCenter(screen, "10", 130);
    
    SDL_UpdateRect(screen, 0, 0, 0, 0);
    
} // FS::GetReadyDisplay::start_draw()
     
// **************************************************

bool FS::GetReadyDisplay::redraw(int gt, KeyboardEvent * events, int n)
{
    int seconds_passed = (gt - start_time) / 1000;
    bool jump_pressed = false;
    int i = 0;
    
    while (i < n)
    {
        if ( events[i] == JUMP_KEY_PRESSED )
        {
            jump_pressed = true;
            break;
        } // if
        ++i;
    } // while
    
    if ( (seconds_passed == SECONDS_TO_WAIT) || jump_pressed )
    {
        //finished = true;
        //return;
        return true;
    } // if
    
    SDL_Rect erase_rect = { 390, 130, 50, 50 };
    SDL_Rect dest_rect = { 390, 130 };
    SDL_BlitSurface(background, &erase_rect, screen, &dest_rect);
    
    char seconds[3];
    sprintf(seconds, "%d", (SECONDS_TO_WAIT - seconds_passed));
    my_font.writeCenter(screen, seconds, 130);
    
    SDL_UpdateRect(screen, 390, 130, 50, 50);
    
    return false;
    
} // FS::GetReadyDisplay::redraw()

// **************************************************
// **************************************************
// **************************************************
// **************************************************
